launch_new_process = False
minimize_to_tray = True
console_line_limit = 1024
start_minimized = False


def load_settings():
    import json
    from xappt_qt.constants import APP_CONFIG_PATH
    config_path = APP_CONFIG_PATH.joinpath("settings.cfg")
    try:
        with open(config_path, "r") as fp:
            settings_raw = json.load(fp)
    except (FileNotFoundError, json.JSONDecodeError):
        pass
    else:
        global console_line_limit
        console_line_limit = settings_raw.get('console_line_limit', 1024)


load_settings()
del load_settings



import xappt

from xappt_qt.utilities import safe_file_name


__all__ = [
    'APP_INTERFACE_NAME',
    'APP_PACKAGE_NAME',
    'APP_TITLE',
    'APP_PROPERTY_RUNNING',
    'APP_PROPERTY_LAUNCHER',
    'APP_CONFIG_PATH',
]


APP_INTERFACE_NAME = "qt"
APP_TITLE = "Xappt QT"
APP_PACKAGE_NAME = safe_file_name(APP_TITLE)
APP_PROPERTY_RUNNING = "running"
APP_PROPERTY_LAUNCHER = "launcher"
APP_CONFIG_PATH = xappt.user_data_path().joinpath(APP_PACKAGE_NAME)



import argparse
import sys

from typing import Dict, List, Type

from PyQt5 import QtWidgets

import xappt
from xappt.models.parameter import convert

from xappt_qt.gui.utilities.dark_palette import apply_palette
from xappt_qt.constants import *


def main(argv) -> int:
    parser = argparse.ArgumentParser()
    parser.add_argument('toolname', help='Specify the name of the tool to load')
    parser.add_argument('--auto-run', action='store_true', help='Automatically run the tool when it is invoked.')

    options, unknowns = parser.parse_known_args(args=argv)

    xappt.discover_plugins()

    tool_class = xappt.get_tool_plugin(options.toolname)
    if tool_class is None:
        raise SystemExit(f"Tool {options.toolname} not found.")

    app = QtWidgets.QApplication.instance() or QtWidgets.QApplication(argv)
    apply_palette(app)

    app.setProperty(APP_PROPERTY_RUNNING, True)
    app.setProperty(APP_PROPERTY_LAUNCHER, True)

    interface = xappt.get_interface()
    params = params_from_args(tool_class=tool_class, tool_args=unknowns)
    tool_instance = tool_class(interface=interface, **params)
    invoke_options = {
        'auto_run': options.auto_run,
        'headless': False,
    }
    if hasattr(tool_instance, "headless") and tool_instance.headless:
        invoke_options['auto_run'] = True
        invoke_options['headless'] = True
    interface.invoke(tool_instance, **invoke_options)

    return app.exec_()


def params_from_args(tool_class: Type[xappt.BaseTool], tool_args: List[str]) -> Dict:
    if not tool_args:
        return {}

    tool_parser = argparse.ArgumentParser()

    for parameter in tool_class.class_parameters():
        setup_args = parameter.param_setup_args
        args, kwargs = convert.to_argument_dict(setup_args)
        kwargs['required'] = False
        tool_parser.add_argument(*args, **kwargs)

    tool_options, _ = tool_parser.parse_known_args(args=tool_args)
    return {k: v for k, v in vars(tool_options).items() if v is not None}


def entry_point() -> int:
    return main(sys.argv[1:])


if __name__ == '__main__':
    sys.exit(entry_point())



from PyQt5 import QtCore, QtWidgets

TEXT_EDIT_STYLE_SHEET = """
QPlainTextEdit {
    border: 1px solid palette(base);
    border-radius: 3px;
    background-color: palette(base);
    padding: 1px;
}
QPlainTextEdit:focus {
    border-color: palette(highlight);
}"""


class TextEdit(QtWidgets.QPlainTextEdit):
    editingFinished = QtCore.pyqtSignal()

    def __init__(self, parent=None):
        super().__init__(parent=parent)
        self._changed = False
        self.setTabChangesFocus(True)
        self.textChanged.connect(self._handle_text_changed)
        self.setStyleSheet(TEXT_EDIT_STYLE_SHEET)
        self.setLineWrapMode(self.NoWrap)

    def focusOutEvent(self, event):
        if self._changed:
            self.editingFinished.emit()
        super().focusOutEvent(event)

    def _handle_text_changed(self):
        self._changed = True

    def setTextChanged(self, state=True):
        self._changed = state

    def setText(self, text: str):
        self.setPlainText(text)
        self._changed = False

    def text(self) -> str:
        return self.toPlainText()



from PyQt5 import QtWidgets

from xappt_qt.constants import *


class ErrorLabel(QtWidgets.QLabel):
    def __init__(self):
        super().__init__()
        self._message: str = ""
        self.linkActivated.connect(self._on_link_activated)

    def reset(self):
        self._message = ""
        self.setText("")

    def set_error(self, message: str):
        self._message = message
        self.setText('<a href="#" style="color: red; text-decoration:none; font-size: 14pt;">⛔</a>')

    def show_error(self):
        if len(self._message):
            QtWidgets.QMessageBox.critical(self.parent(), APP_TITLE, self._message)

    def _on_link_activated(self, _: str):
        self.show_error()



from PyQt5 import QtCore, QtGui

from xappt_qt.gui.ui.browser_tab_options import Ui_tabOptions
from xappt_qt.gui.tab_pages.base import BaseTabPage
import xappt_qt.config


class OptionsTabPage(BaseTabPage, Ui_tabOptions):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.setupUi(self)
        self.chkLaunchNewProcess.stateChanged.connect(self.on_launch_new_process_changed)
        self.chkMinimizeToTray.stateChanged.connect(self.on_minimize_to_tray_changed)
        self.chkStartMinimized.stateChanged.connect(self.on_start_minimized_changed)

    @staticmethod
    def on_launch_new_process_changed(new_state: int):
        xappt_qt.config.launch_new_process = new_state == QtCore.Qt.Checked

    @staticmethod
    def on_minimize_to_tray_changed(new_state: int):
        xappt_qt.config.minimize_to_tray = new_state == QtCore.Qt.Checked

    @staticmethod
    def on_start_minimized_changed(new_state: int):
        xappt_qt.config.start_minimized = new_state == QtCore.Qt.Checked

    def apply_settings(self):
        self.chkLaunchNewProcess.setChecked(xappt_qt.config.launch_new_process)
        self.chkMinimizeToTray.setChecked(xappt_qt.config.minimize_to_tray)
        self.chkStartMinimized.setChecked(xappt_qt.config.start_minimized)

    def showEvent(self, event: QtGui.QShowEvent):
        super().showEvent(event)
        self.apply_settings()
